package dev.fyloz.colorrecipesexplorer.logic

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.LogicComponent
import dev.fyloz.colorrecipesexplorer.dtos.CompanyDto
import dev.fyloz.colorrecipesexplorer.service.CompanyService

interface CompanyLogic : Logic<CompanyDto, CompanyService>

@LogicComponent
class DefaultCompanyLogic(service: CompanyService) :
    BaseLogic<CompanyDto, CompanyService>(service, Constants.ModelNames.COMPANY), CompanyLogic {
    override fun save(dto: CompanyDto): CompanyDto {
        throwIfNameAlreadyExists(dto.name)

        return super.save(dto)
    }

    override fun update(dto: CompanyDto): CompanyDto {
        throwIfNameAlreadyExists(dto.name, dto.id)

        return super.update(dto)
    }

    override fun deleteById(id: Long) {
        if (service.isUsedByRecipe(id)) {
            throw cannotDeleteException("Cannot delete the company with the id '$id' because one or more recipes depends on it")
        }

        super.deleteById(id)
    }

    private fun throwIfNameAlreadyExists(name: String, id: Long? = null) {
        if (service.existsByName(name, id)) {
            throw alreadyExistsException(value = name)
        }
    }
}