package dev.fyloz.colorrecipesexplorer.logic

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.LogicComponent
import dev.fyloz.colorrecipesexplorer.dtos.GroupDto
import dev.fyloz.colorrecipesexplorer.dtos.RecipeGroupInformationDto
import dev.fyloz.colorrecipesexplorer.dtos.RecipeStepDto
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionError
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionsException
import dev.fyloz.colorrecipesexplorer.exception.RestException
import dev.fyloz.colorrecipesexplorer.model.account.Group
import dev.fyloz.colorrecipesexplorer.service.RecipeStepService
import dev.fyloz.colorrecipesexplorer.utils.PositionUtils
import org.springframework.http.HttpStatus

interface RecipeStepLogic : Logic<RecipeStepDto, RecipeStepService> {
    /** Validates the steps of the given [groupInformation], according to the criteria of [PositionUtils.validate]. */
    fun validateGroupInformationSteps(groupInformation: RecipeGroupInformationDto)
}

@LogicComponent
class DefaultRecipeStepLogic(recipeStepService: RecipeStepService) :
    BaseLogic<RecipeStepDto, RecipeStepService>(recipeStepService, Constants.ModelNames.RECIPE_STEP), RecipeStepLogic {
    override fun validateGroupInformationSteps(groupInformation: RecipeGroupInformationDto) {
        try {
            PositionUtils.validate(groupInformation.steps.map { it.position }.toList())
        } catch (ex: InvalidPositionsException) {
            throw InvalidGroupStepsPositionsException(groupInformation.group, ex)
        }
    }
}

class InvalidGroupStepsPositionsException(
    val group: GroupDto,
    val exception: InvalidPositionsException
) : RestException(
    "invalid-groupinformation-recipestep-position",
    "Invalid steps positions",
    HttpStatus.BAD_REQUEST,
    "The position of steps for the group ${group.name} are invalid",
    mapOf(
        "group" to group.name,
        "groupId" to group.id,
        "invalidSteps" to exception.errors
    )
) {
    val errors: Set<InvalidPositionError>
        get() = exception.errors
}