package dev.fyloz.colorrecipesexplorer.logic

import dev.fyloz.colorrecipesexplorer.dtos.MaterialDto
import dev.fyloz.colorrecipesexplorer.dtos.MaterialTypeDto
import dev.fyloz.colorrecipesexplorer.dtos.MixMaterialDto
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionError
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionsException
import dev.fyloz.colorrecipesexplorer.utils.PositionUtils
import io.mockk.*
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows

class DefaultMixQuantityLogicTest {
    private val materialLogicMock = mockk<MaterialLogic>()
    private val mixTypeLogicMock = mockk<MixTypeLogic>()

    private val mixMaterialLogic = DefaultMixQuantityLogic(materialLogicMock, mixTypeLogicMock)

    @AfterEach
    internal fun afterEach() {
        clearAllMocks()
    }

    @Test
    fun validateMixMaterials_normalBehavior_doesNothing() {
        // Arrange
        val materialType = MaterialTypeDto(1L, "Unit test material type", "UTMT", false)
        val material = MaterialDto(1L, "Unit test material", 1000f, false, materialType)
        val mixMaterial = MixMaterialDto(1L, material, 100f, 1)

        mockkObject(PositionUtils)
        every { PositionUtils.validate(any()) } just runs

        // Act
        // Assert
        mixMaterialLogic.validateMixQuantities(listOf(mixMaterial))
    }

    @Test
    fun validateMixMaterials_emptySet_doesNothing() {
        // Arrange
        // Act
        // Assert
        mixMaterialLogic.validateMixQuantities(listOf())
    }

    @Test
    fun validateMixMaterials_firstUsesPercents_throwsInvalidFirstMixMaterialException() {
        // Arrange
        val materialType = MaterialTypeDto(1L, "Unit test material type", "UTMT", true)
        val material = MaterialDto(1L, "Unit test material", 1000f, false, materialType)
        val mixMaterial = MixMaterialDto(1L, material, 100f, 1)

        mockkObject(PositionUtils)
        every { PositionUtils.validate(any()) } just runs

        // Act
        // Assert
        assertThrows<InvalidFirstMixMaterialException> { mixMaterialLogic.validateMixQuantities(listOf(mixMaterial)) }
    }

    @Test
    fun validateMixMaterials_invalidPositions_throwsInvalidMixMaterialsPositionsException() {
        // Arrange
        val materialType = MaterialTypeDto(1L, "Unit test material type", "UTMT", false)
        val material = MaterialDto(1L, "Unit test material", 1000f, false, materialType)
        val mixMaterial = MixMaterialDto(1L, material, 100f, 1)

        val errors = setOf(InvalidPositionError("error", "An unit test error"))

        mockkObject(PositionUtils)
        every { PositionUtils.validate(any()) } throws InvalidPositionsException(errors)

        // Act
        // Assert
        assertThrows<InvalidMixMaterialsPositionsException> { mixMaterialLogic.validateMixQuantities(listOf(mixMaterial)) }
    }
}